(*
  Copyright 2010 Gerd Stolpmann

  This file is part of Plasma, a distributed filesystem and a
  map/reduce computation framework. Unless you have a written license
  agreement with the copyright holder (Gerd Stolpmann), the following
  terms apply:

  Plasma is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Plasma is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.

*)
let esys = Unixqueue.create_unix_event_system()

let c =
  Plasma_client.open_cluster 
    "test"  [ "office3", 2730; "office4", 2730 ] esys

let input = "/work/mapped_000021"

let input_blocks = Mapred_io.file_blocks c input
let reader =
  if input_blocks = 0L then
    Mapred_io.read_multiple []
  else
    Mapred_io.read_file c input 0L input_blocks

let l = ref []
let n = ref 0;;
( try
    while true do
      let line = reader # input_record() in
      let key = Mapred_split.tab_split_key line in
      let hash = Hashtbl.hash key in
      l := (hash,key,line) :: !l;
      incr n
    done
  with
    | End_of_file -> ()
);;
let a = Array.make !n (0,"","")
let k = ref (!n-1);;
while !l <> [] do
  a.( !k ) <- List.hd !l;
  l := List.tl !l;
  decr k
done;
assert(!k = (-1));

Array.stable_sort
  (fun (h1,k1,l1) (h2,k2,l2) ->
     if h1 = h2 then
       String.compare k1 k2
     else
       h1 - h2   (* h1, h2 are both positive *)
  )
  a
